import os

from hokusai.lib.command import command
from hokusai.lib.common import print_green

@command
def version():
  version_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), os.path.pardir, 'VERSION.txt')
  print_green(open(version_file, 'r').read())



import os
import json

from hokusai.lib.config import config
from hokusai.lib.common import shout, returncode, k8s_uuid
from hokusai.services.kubectl import Kubectl

class CommandRunner(object):
  def __init__(self, context):
    self.context = context
    self.kctl = Kubectl(self.context)

  def run(self, image_tag, cmd, tty=False, env=()):
    if os.environ.get('USER') is not None:
      uuid = "%s-%s" % (os.environ.get('USER'), k8s_uuid())
    else:
      uuid = k8s_uuid()

    name = "%s-hokusai-run-%s" % (config.project_name, uuid)
    image_name = "%s:%s" % (config.aws_ecr_registry, image_tag)
    container = {
      "args": cmd.split(' '),
      "name": name,
      "image": image_name,
      "imagePullPolicy": "Always",
      'envFrom': [{'configMapRef': {'name': "%s-environment" % config.project_name}}]
    }

    if tty:
      container.update({
        "stdin": True,
        "stdinOnce": True,
        "tty": True
      })

    if len(env):
      container['env'] = []
      for s in env:
        if '=' not in s:
          print_red("Error: environment variables must be of the form 'KEY=VALUE'")
          return -1
        split = s.split('=', 1)
        container['env'].append({'name': split[0], 'value': split[1]})

    overrides = {
      "apiVersion": "v1",
      "spec": {
        "containers": [container]
      }
    }

    if tty:
      shout(self.kctl.command("run %s -t -i --image=%s --restart=Never --overrides='%s' --rm" %
                     (name, image_name, json.dumps(overrides))), print_output=True)
    else:
      return returncode(self.kctl.command("run %s --attach --image=%s --overrides='%s' --restart=Never --rm" %
                                        (name, image_name, json.dumps(overrides))))



import sys
import traceback
from functools import wraps

from hokusai.lib.common import print_red, CalledProcessError, get_verbosity

def command(func):
  @wraps(func)
  def wrapper(*args, **kwargs):
    try:
      result = func(*args, **kwargs)
      if result is None:
        sys.exit(0)
      else:
        sys.exit(result)
    except SystemExit:
      raise
    except KeyboardInterrupt:
      raise
    except CalledProcessError, e:
      if get_verbosity():
        print_red(traceback.format_exc(e))
      else:
        print_red("ERROR: %s" % str(e))
      sys.exit(-1)
    except Exception, e:
      if get_verbosity():
        print_red(traceback.format_exc(e))
      else:
        print_red("ERROR: %s" % str(e))
      sys.exit(-1)
  return wrapper



import datetime
import json

from hokusai.lib.config import config
from hokusai.services.kubectl import Kubectl
from hokusai.services.ecr import ECR
from hokusai.lib.common import print_red, print_green, shout

class Deployment(object):
  def __init__(self, context):
    self.context = context
    self.kctl = Kubectl(self.context)
    self.cache = self.kctl.get_object('deployment', selector="app=%s,layer=application" % config.project_name)

  def update(self, tag):
    print_green("Deploying %s to %s..." % (tag, self.context))

    if self.context != tag:
      ecr = ECR()

      ecr.retag(tag, self.context)
      print_green("Updated tag %s -> %s" % (tag, self.context))

      deployment_tag = "%s--%s" % (self.context, datetime.datetime.utcnow().strftime("%Y-%m-%d--%H-%M-%S"))
      ecr.retag(tag, deployment_tag)
      print_green("Updated tag %s -> %s" % (tag, deployment_tag))

    deployment_timestamp = datetime.datetime.utcnow().strftime("%s%f")
    for deployment in self.cache:
      containers = deployment['spec']['template']['spec']['containers']
      container_names = [container['name'] for container in containers]
      deployment_targets = [{"name": name, "image": "%s:%s" % (config.aws_ecr_registry, tag)} for name in container_names]
      patch = {
        "spec": {
          "template": {
            "metadata": {
              "labels": {"deploymentTimestamp": deployment_timestamp}
            },
            "spec": {
              "containers": deployment_targets
            }
          }
        }
      }
      print_green("Updating %s..." % deployment['metadata']['name'])
      shout(self.kctl.command("patch deployment %s -p '%s'" % (deployment['metadata']['name'], json.dumps(patch))))

  def refresh(self):
    deployment_timestamp = datetime.datetime.utcnow().strftime("%s%f")
    for deployment in self.cache:
      patch = {
        "spec": {
          "template": {
            "metadata": {
              "labels": {"deploymentTimestamp": deployment_timestamp}
            }
          }
        }
      }
      print_green("Refreshing %s..." % deployment['metadata']['name'])
      shout(self.kctl.command("patch deployment %s -p '%s'" % (deployment['metadata']['name'], json.dumps(patch))))

  def history(self, deployment_name):
    replicasets = self.kctl.get_object('replicaset', selector="app=%s,layer=application" % config.project_name)
    replicasets = filter(lambda rs: rs['metadata']['ownerReferences'][0]['name'] == deployment_name, replicasets)
    return sorted(replicasets, key=lambda rs: int(rs['metadata']['annotations']['deployment.kubernetes.io/revision']))

  @property
  def names(self):
    return [deployment['metadata']['name'] for deployment in self.cache]

  @property
  def current_tag(self):
    images = []
    for deployment in self.cache:
      containers = deployment['spec']['template']['spec']['containers']
      container_names = [container['name'] for container in containers]
      container_images = [container['image'] for container in containers]
      if not all(x == container_images[0] for x in container_images):
        print_red("Deployment's containers do not reference the same image tag")
        return None
      images.append(containers[0]['image'])
    if not all(y == images[0] for y in images):
      print_red("Deployments do not reference the same image tag")
      return None
    return images[0].rsplit(':', 1)[1]



import os
import sys

from collections import OrderedDict
from tempfile import NamedTemporaryFile

import yaml

from hokusai.lib.config import config
from hokusai.lib.common import print_red, print_green, shout
from hokusai.services.kubectl import Kubectl

class ConfigMap(object):
  def __init__(self, context):
    self.context = context
    self.kctl = Kubectl(context)
    self.struct = OrderedDict([
      ('apiVersion', 'v1'),
      ('kind', 'ConfigMap'),
      ('metadata', {
        'labels': {'app': config.project_name},
        'name': "%s-environment" % config.project_name
      }),
      ('data', {})
    ])

  def _to_file(self):
    f = NamedTemporaryFile(delete=False)
    f.write(yaml.safe_dump(self.struct, default_flow_style=False))
    f.close()
    return f.name

  def create(self):
    f = self._to_file()
    try:
      shout(self.kctl.command("create -f %s" % f))
    finally:
      os.unlink(f)

  def destroy(self):
    shout(self.kctl.command("delete configmap %s-environment" % config.project_name))

  def load(self):
    payload = shout(self.kctl.command("get configmap %s-environment -o yaml" % config.project_name))
    struct = yaml.load(payload)
    if 'data' in struct:
      self.struct['data'] = struct['data']
    else:
      self.struct['data'] = {}

  def save(self):
    f = self._to_file()
    try:
      shout(self.kctl.command("apply -f %s" % f))
    finally:
      os.unlink(f)

  def all(self):
    for k, v in self.struct['data'].iteritems():
      yield k, v

  def update(self, key, value):
    self.struct['data'].update({key: value})

  def delete(self, key):
    try:
      del self.struct['data'][key]
    except KeyError:
      print_red("Cannot unset '%s' as it does not exist" % key)



from hokusai.lib.command import command
from hokusai.lib.common import print_green
from hokusai.services.deployment import Deployment

@command
def history(context):
  deployment = Deployment(context)
  for name in deployment.names:
    print_green(name)
    print("REVISION    CREATION TIMESTAMP            CONTAINER NAME - IMAGE TAG")
    for replicaset in deployment.history(name):
      revision = replicaset['metadata']['annotations']['deployment.kubernetes.io/revision']
      created_at = replicaset['metadata']['creationTimestamp']
      containers = ["%s - %s" % (container['name'], container['image'].rsplit(':', 1)[1]) for container in replicaset['spec']['template']['spec']['containers']]
      print("%s           %s          %s" % (revision, created_at, ','.join(containers)))



import json

import yaml

from hokusai.lib.common import shout

class Kubectl(object):
  def __init__(self, context):
    self.context = context

  def command(self, cmd):
    return "kubectl --context %s %s" % (self.context, cmd)

  def get_object(self, obj, selector=None):
    if selector is not None:
      cmd = self.command("get %s --selector %s -o json" % (obj, selector))
    else:
      cmd = self.command("get %s -o json" % obj)
    try:
      return json.loads(shout(cmd))['items']
    except ValueError:
      return []

  def contexts(self):
    return [context['name'] for context in yaml.load(shout('kubectl config view'))['contexts']]



from hokusai.lib.command import command
from hokusai.services.command_runner import CommandRunner

@command
def run(context, cmd, tty, tag, env):
  if tag is not None:
    image_tag = tag
  else:
    image_tag = context

  return CommandRunner(context).run(image_tag, cmd, tty=tty, env=env)



import signal
import string
import random
import json

from collections import OrderedDict

from subprocess import call, check_call, check_output, CalledProcessError, STDOUT

import yaml
import boto3

from termcolor import cprint

EXIT_SIGNALS = [signal.SIGHUP, signal.SIGINT, signal.SIGQUIT, signal.SIGPIPE, signal.SIGTERM]

YAML_HEADER = '---\n'

VERBOSE = False

def print_green(msg):
  cprint(msg, 'green')

def print_red(msg):
  cprint(msg, 'red')

def set_verbosity(v):
  global VERBOSE
  VERBOSE = v

def get_verbosity():
  global VERBOSE
  return VERBOSE

def verbose(msg):
  if VERBOSE: cprint("==> hokusai exec `%s`" % msg, 'yellow')
  return msg

def returncode(command):
  return call(verbose(command), stderr=STDOUT, shell=True)

def shout(command, print_output=False):
  if print_output:
    return check_call(verbose(command), stderr=STDOUT, shell=True)
  else:
    return check_output(verbose(command), stderr=STDOUT, shell=True)

def k8s_uuid():
  uuid = []
  for i in range(0,5):
    uuid.append(random.choice(string.lowercase))
  return ''.join(uuid)

def build_deployment(name, image, target_port, layer='application', component='web', environment=None, always_pull=False, replicas=1):
  container = {
    'name': "%s-%s" % (name, component),
    'image': image,
    'ports': [{'containerPort': target_port}]
  }

  if layer == 'application':
    container['envFrom'] = [{'configMapRef': {'name': "%s-environment" % name}}]

  if environment is not None:
    container['env'] = environment

  if always_pull:
    container['imagePullPolicy'] = 'Always'

  deployment = OrderedDict([
    ('apiVersion', 'extensions/v1beta1'),
    ('kind', 'Deployment'),
    ('metadata', {'name': "%s-%s" % (name, component)}),
    ('spec', {
      'replicas': replicas,
      'strategy': {
        'rollingUpdate': {
          'maxSurge': 1,
          'maxUnavailable': 0
        },
        'type': 'RollingUpdate'
      },
      'template': {
        'metadata': {
          'labels': {
            'app': name,
            'layer': layer,
            'component': component
            },
            'name': "%s-%s" % (name, component),
            'namespace': 'default'
          },
          'spec': {
            'containers': [container]
          }
        }
      }
    )
  ])
  return YAML_HEADER + yaml.safe_dump(deployment, default_flow_style=False)

def build_service(name, port, layer='application', component='web', target_port=None, internal=True):
  if target_port is None:
    target_port = port

  spec = {
    'ports': [{'port': port, 'targetPort': target_port, 'protocol': 'TCP'}],
    'selector': {
      'app': name,
      'layer': layer,
      'component': component
    }
  }

  if internal:
    spec['type'] = 'ClusterIP'
  else:
    spec['type'] = 'LoadBalancer'
    spec['sessionAffinity'] = 'None'

  service = OrderedDict([
    ('apiVersion', 'v1'),
    ('kind', 'Service'),
    ('metadata', {
      'labels': {
        'app': name,
        'layer': layer,
        'component': component
      },
      'name': "%s-%s" % (name, component),
      'namespace': 'default'
    }),
    ('spec', spec)
  ])
  return YAML_HEADER + yaml.safe_dump(service, default_flow_style=False)



import os
import signal

from hokusai.lib.command import command
from hokusai.lib.config import config
from hokusai.lib.common import print_red, print_green, shout, EXIT_SIGNALS, CalledProcessError

@command
def test():
  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/test.yml')
  if not os.path.isfile(docker_compose_yml):
    print_red("Yaml file %s does not exist." % docker_compose_yml)
    return -1

  def cleanup(*args):
    shout("docker-compose -f %s -p ci stop" % docker_compose_yml, print_output=True)
  for sig in EXIT_SIGNALS:
    signal.signal(sig, cleanup)

  shout("docker-compose -f %s -p ci up --build --abort-on-container-exit" % docker_compose_yml, print_output=True)

  try:
    test_exit_code = int(shout("docker wait ci_%s_1" % config.project_name))
  except CalledProcessError:
    print_red('Docker wait failed.')
    shout("docker-compose -f %s -p ci stop" % docker_compose_yml, print_output=True)
    return -1

  if test_exit_code != 0:
    print_red('Tests Failed - Exit Code: %s\n' % test_exit_code)
  else:
    print_green("Tests Passed")

  shout("docker-compose -f %s -p ci stop" % docker_compose_yml, print_output=True)

  return test_exit_code



from hokusai.lib.config import config
from hokusai.services.kubectl import Kubectl

class Service(object):
  def __init__(self, context):
    self.context = context
    self.kctl = Kubectl(self.context)
    self.cache = self.kctl.get_object('service', selector="app=%s,layer=application" % config.project_name)



import os
import urllib
import shutil

from distutils.dir_util import mkpath

import boto3

from hokusai.lib.command import command
from hokusai.lib.common import print_red, print_green

@command
def configure(kubectl_version, bucket_name, key_name, platform, install_to, install_config_to):
  print_green("Downloading and installing kubectl...")
  urllib.urlretrieve("https://storage.googleapis.com/kubernetes-release/release/v%s/bin/%s/amd64/kubectl" % (kubectl_version, platform), os.path.join('/tmp', 'kubectl'))
  os.chmod(os.path.join('/tmp', 'kubectl'), 0755)
  shutil.move(os.path.join('/tmp', 'kubectl'), os.path.join(install_to, 'kubectl'))

  print_green("Configuring kubectl...")
  if not os.path.isdir(install_config_to):
    mkpath(install_config_to)

  bucket = boto3.resource('s3').Bucket(bucket_name)
  bucket.download_file(key_name, os.path.join(install_config_to, 'config'))



from hokusai.lib.config import config
from hokusai.lib.command import command
from hokusai.services.configmap import ConfigMap
from hokusai.lib.common import print_red, print_green

@command
def create_env(context):
  configmap = ConfigMap(context)
  configmap.create()
  print_green("Created configmap %s-environment" % config.project_name)

@command
def delete_env(context):
  configmap = ConfigMap(context)
  configmap.destroy()
  print_green("Deleted configmap %s-environment" % config.project_name)

@command
def get_env(context, environment):
  configmap = ConfigMap(context)
  configmap.load()
  if len(environment) == 0:
    for k, v in configmap.all():
      print("%s=%s" % (k, v))
  else:
    for k, v in configmap.all():
      if k in environment:
        print("%s=%s" % (k, v))

@command
def set_env(context, environment):
  configmap = ConfigMap(context)
  configmap.load()
  for s in environment:
    if '=' not in s:
      print_red("Error: environment variables must be of the form 'KEY=VALUE'")
      return -1
    split = s.split('=', 1)
    configmap.update(split[0], split[1])
  configmap.save()

@command
def unset_env(context, environment):
  configmap = ConfigMap(context)
  configmap.load()
  for s in environment:
    configmap.delete(s)
  configmap.save()



import re
import base64

import boto3
from botocore.exceptions import BotoCoreError, ClientError

from hokusai.lib.config import config

SHA1_REGEX = re.compile(r"\b[0-9a-f]{40}\b")

class ECR(object):
  def __init__(self):
    self.client = boto3.client('ecr', region_name=config.aws_ecr_region)

  def registry(self):
    repos = []
    res = self.client.describe_repositories(registryId=config.aws_account_id)
    repos += res['repositories']
    while 'nextToken' in res:
      res = self.client.describe_repositories(registryId=self.registry_id,
                                                nextToken=res['nextToken'])
      repos += res['repositories']
    return repos

  def project_repository_exists(self):
    repo_names = [r['repositoryName'] for r in self.registry()]
    return config.project_name in repo_names

  def create_project_repository(self):
    try:
      repository = self.client.create_repository(repositoryName=config.project_name)['repository']
    except BotoCoreError:
      return False
    except KeyError:
      return False
    try:
      assert 'createdAt' in repository
      assert repository['registryId'] == config.aws_account_id
      assert 'repositoryArn' in repository
      assert repository['repositoryName'] == config.project_name
      assert 'repositoryUri' in repository
    except AssertionError:
      return False
    return True

  def get_login(self):
    res = self.client.get_authorization_token(registryIds=[str(config.aws_account_id)])['authorizationData'][0]
    token = base64.b64decode(res['authorizationToken'])
    username = token.split(':')[0]
    password = token.split(':')[1]
    return "docker login -u %s -p %s %s" % (username, password, res['proxyEndpoint'])

  def get_image_by_tag(self, tag):
    res = self.client.describe_images(registryId=config.aws_account_id,
                                      repositoryName=config.project_name,
                                      imageIds=[{'imageTag': tag}])
    return res['imageDetails'][0]

  def get_images(self):
    images = []
    res = self.client.describe_images(registryId=config.aws_account_id,
                                  repositoryName=config.project_name)
    images += res['imageDetails']
    while 'nextToken' in res:
      res = self.client.describe_images(registryId=config.aws_account_id,
                                    repositoryName=config.project_name,
                                    nextToken=res['nextToken'])
      images += res['imageDetails']
    return images

  def tag_exists(self, tag):
    for image in self.get_images():
      if 'imageTags' not in image.keys():
        continue
      if tag in image['imageTags']:
        return True
    return False

  def find_git_sha1_image_tag(self, tag):
    image = self.get_image_by_tag(tag)
    for t in image['imageTags']:
      if SHA1_REGEX.match(t) is not None:
        return t
    return None

  def retag(self, tag, new_tag):
    res = self.client.batch_get_image(
      registryId=config.aws_account_id,
      repositoryName=config.project_name,
      imageIds=[{'imageTag': tag}]
    )

    if len(res['failures']) and not len(res['images']):
      raise ValueError("Failed to retrieve image manifest for tag %s" % tag)

    image = res['images'][0]

    try:
      self.client.put_image(
          registryId=config.aws_account_id,
          repositoryName=config.project_name,
          imageManifest=image['imageManifest'],
          imageTag=new_tag
      )
    except ClientError as e:
      if e.response['Error']['Code'] != 'ImageAlreadyExistsException':
        raise



import os
import sys

from collections import OrderedDict

import yaml

from hokusai.lib.common import print_red, YAML_HEADER

HOKUSAI_CONFIG_FILE = os.path.join(os.getcwd(), 'hokusai', 'config.yml')

class HokusaiConfigError(Exception):
  pass

class HokusaiConfig(object):
  def create(self, project_name, aws_account_id, aws_ecr_region):
    config = OrderedDict([
      ('aws-account-id', aws_account_id),
      ('aws-ecr-region', aws_ecr_region),
      ('project-name', project_name)
    ])

    with open(HOKUSAI_CONFIG_FILE, 'w') as f:
      payload = YAML_HEADER + yaml.safe_dump(config, default_flow_style=False)
      f.write(payload)

    return self

  def check(self):
    if not os.path.isfile(HOKUSAI_CONFIG_FILE):
      raise HokusaiConfigError("Hokusai is not configured for this project - run 'hokusai configure'")
    return self

  def get(self, key):
    self.check()
    config_file = open(HOKUSAI_CONFIG_FILE, 'r')
    config_data = config_file.read()
    config_file.close()
    config = yaml.safe_load(config_data)
    try:
      return config[key]
    except KeyError:
      return None

  @property
  def project_name(self):
    return self.get('project-name')

  @property
  def aws_account_id(self):
    return str(self.get('aws-account-id'))

  @property
  def aws_ecr_region(self):
    return self.get('aws-ecr-region')

  @property
  def aws_ecr_registry(self):
    return "%s.dkr.ecr.%s.amazonaws.com/%s" % (self.aws_account_id, self.aws_ecr_region, self.project_name)

config = HokusaiConfig()



import os

from hokusai.lib.command import command
from hokusai.lib.config import config
from hokusai.services.ecr import ECR
from hokusai.services.kubectl import Kubectl
from hokusai.lib.common import print_red, print_green, shout, CalledProcessError

@command
def check():
  return_code = 0

  def check_ok(check_item):
    print_green(u'\u2714 ' + check_item + ' found')

  def check_err(check_item):
    print_red(u'\u2718 ' + check_item + ' not found')

  config.check()

  try:
    shout('docker --version')
    check_ok('docker')
  except CalledProcessError:
    check_err('docker')
    return_code += 1

  try:
    shout('docker-compose --version')
    check_ok('docker-compose')
  except CalledProcessError:
    check_err('docker-compose')
    return_code += 1

  try:
    shout('kubectl version')
    check_ok('kubectl')
  except CalledProcessError:
    check_err('kubectl')
    return_code += 1

  try:
    shout('git version')
    check_ok('git')
  except CalledProcessError:
    check_err('git')
    return_code += 1

  if os.environ.get('AWS_ACCESS_KEY_ID') is not None:
    check_ok('AWS_ACCESS_KEY_ID')
  else:
    check_err('AWS_ACCESS_KEY_ID')
    return_code += 1

  if os.environ.get('AWS_SECRET_ACCESS_KEY') is not None:
    check_ok('AWS_SECRET_ACCESS_KEY')
  else:
    check_err('AWS_SECRET_ACCESS_KEY')
    return_code += 1

  if os.environ.get('AWS_DEFAULT_REGION') is not None:
    check_ok('AWS_DEFAULT_REGION')
  else:
    check_err('AWS_DEFAULT_REGION')
    return_code += 1

  if os.environ.get('AWS_ACCOUNT_ID') is not None:
    check_ok('AWS_ACCOUNT_ID')
  else:
    check_err('AWS_ACCOUNT_ID')
    return_code += 1

  ecr = ECR()
  if ecr.project_repository_exists():
    check_ok("ECR repository '%s'" % config.project_name)
  else:
    check_err("ECR repository '%s'" % config.project_name)
    return_code += 1

  if os.path.isfile(os.path.join(os.getcwd(), 'hokusai/common.yml')):
    check_ok('hokusai/common.yml')
  else:
    check_err('hokusai/common.yml')
    return_code += 1

  if os.path.isfile(os.path.join(os.getcwd(), 'hokusai/development.yml')):
    check_ok('hokusai/development.yml')
  else:
    check_err('hokusai/development.yml')
    return_code += 1

  if os.path.isfile(os.path.join(os.getcwd(), 'hokusai/test.yml')):
    check_ok('hokusai/test.yml')
  else:
    check_err('hokusai/test.yml')
    return_code += 1

  for context in ['staging', 'production']:
    if context in Kubectl('staging').contexts():
      check_ok("kubectl context '%s'" % context)
    else:
      check_err("kubectl context '%s'" % context)
      return_code += 1

    if os.path.isfile(os.path.join(os.getcwd(), "hokusai/%s.yml" % context)):
      check_ok("hokusai/%s.yml" % context)
    else:
      check_err("hokusai/%s.yml" % context)
      return_code += 1

  return return_code



import os

from hokusai.lib.command import command
from hokusai.lib.config import config
from hokusai.lib.common import print_red, shout

@command
def dev_start(skip_build, follow):
  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/development.yml')
  if not os.path.isfile(docker_compose_yml):
    print_red("Yaml file %s does not exist." % docker_compose_yml)
    return -1

  opts = ''
  if not skip_build:
    opts += ' --build'
  if not follow:
    opts += ' -d'

  shout("docker-compose -f %s up%s" % (docker_compose_yml, opts), print_output=True)

@command
def dev_stop():
  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/development.yml')
  if not os.path.isfile(docker_compose_yml):
    print_red("Yaml file %s does not exist." % docker_compose_yml)
    return -1

  shout("docker-compose -f %s stop" % docker_compose_yml, print_output=True)

@command
def dev_build():
  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/common.yml')
  if not os.path.isfile(docker_compose_yml):
    print_red("Yaml file %s does not exist." % docker_compose_yml)
    return -1
  shout("docker-compose -f %s build" % docker_compose_yml, print_output=True)

@command
def dev_status():
  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/development.yml')
  if not os.path.isfile(docker_compose_yml):
    print_red("Yaml file %s does not exist." % docker_compose_yml)
    return -1
  shout("docker-compose -f %s ps" % docker_compose_yml, print_output=True)

@command
def dev_logs(follow):
  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/development.yml')
  if not os.path.isfile(docker_compose_yml):
    print_red("Yaml file %s does not exist." % docker_compose_yml)
    return -1

  opts = ''
  if follow:
    opts += ' --follow'
  shout("docker-compose -f %s logs%s" % (docker_compose_yml, opts), print_output=True)

@command
def dev_shell():
  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/development.yml')
  if not os.path.isfile(docker_compose_yml):
    print_red("Yaml file %s does not exist." % docker_compose_yml)
    return -1
  shout("docker-compose -f %s exec %s sh" % (docker_compose_yml, config.project_name), print_output=True)

@command
def dev_clean():
  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/development.yml')
  if not os.path.isfile(docker_compose_yml):
    print_red("Yaml file %s does not exist." % docker_compose_yml)
    return -1
  shout("docker-compose -f %s stop" % docker_compose_yml, print_output=True)
  shout("docker-compose -f %s rm --force" % docker_compose_yml, print_output=True)



from hokusai.lib.command import command
from hokusai.lib.common import print_red, print_green, shout
from hokusai.services.deployment import Deployment
from hokusai.services.ecr import ECR

@command
def diff():
  ecr = ECR()

  staging_deployment = Deployment('staging')
  staging_tag = staging_deployment.current_tag
  if staging_tag is None:
    return -1
  if staging_tag == 'staging':
    staging_tag = ecr.find_git_sha1_image_tag('staging')
    if staging_tag is None:
      print_red("Cound not find a git SHA1 tag for 'staging'.  Aborting.")
      return -1

  production_deployment = Deployment('production')
  production_tag = production_deployment.current_tag
  if production_tag is None:
    return -1
  if production_tag == 'production':
    production_tag = ecr.find_git_sha1_image_tag('production')
    if production_tag is None:
      print_red("Cound not find a git SHA1 tag for 'production'.  Aborting.")
      return -1

  print_green("Comparing %s to %s" % (production_tag, staging_tag))
  shout("git diff %s %s" % (production_tag, staging_tag), print_output=True)



from operator import itemgetter

from hokusai.lib.command import command
from hokusai.services.ecr import ECR
from hokusai.lib.config import config
from hokusai.lib.common import print_green, shout

@command
def images():
  images = ECR().get_images()
  print('')
  print_green('Image Pushed At           | Image Digest                                                     | Image Tags')
  print_green('-------------------------------------------------------------------------------------------------------------------------------')
  for image in sorted(images, key=itemgetter('imagePushedAt'), reverse=True):
    if 'imageTags' not in image.keys():
      continue
    print("%s | %s | %s" % (image['imagePushedAt'], image['imageDigest'][7:], ', '.join(image['imageTags'])))



from hokusai.lib.command import command
from hokusai.lib.common import print_red, print_green
from hokusai.services.deployment import Deployment
from hokusai.services.command_runner import CommandRunner

@command
def promote(migration):
  deploy_from = Deployment('staging')
  tag = deploy_from.current_tag
  if tag is None:
    return -1

  if migration is not None:
    retval = CommandRunner('production').run(tag, migration)
    if retval != 0:
      print_red("Migration failed with return code %s" % retval)
      return retval

  deploy_to = Deployment('production')
  deploy_to.update(tag)
  print_green("Promoted staging to production from %s" % tag)



from hokusai.lib.command import command
from hokusai.lib.common import print_red, print_green
from hokusai.services.deployment import Deployment
from hokusai.services.command_runner import CommandRunner

@command
def deploy(context, tag, migration):
  if migration is not None:
    retval = CommandRunner(context).run(tag, migration)
    if retval != 0:
      print_red("Migration failed with return code %s" % retval)
      return retval

  deployment = Deployment(context)
  deployment.update(tag)
  print_green("Deployment updated to %s" % tag)



from hokusai.commands.check import check
from hokusai.commands.configure import configure
from hokusai.commands.deploy import deploy
from hokusai.commands.development import dev_start, dev_stop, dev_build, dev_status, dev_logs, dev_shell, dev_clean
from hokusai.commands.diff import diff
from hokusai.commands.env import create_env, delete_env, get_env, set_env, unset_env
from hokusai.commands.history import history
from hokusai.commands.images import images
from hokusai.commands.logs import logs
from hokusai.commands.promote import promote
from hokusai.commands.push import push
from hokusai.commands.refresh import refresh
from hokusai.commands.run import run
from hokusai.commands.setup import setup
from hokusai.commands.stack import stack_create, stack_update, stack_delete, stack_status
from hokusai.commands.test import test
from hokusai.commands.version import version



import os

from hokusai.lib.command import command
from hokusai.lib.config import config
from hokusai.services.ecr import ECR
from hokusai.lib.common import print_red, print_green, shout

@command
def push(tag, force, overwrite):
  if force is None and shout('git status --porcelain'):
    print_red("Working directory is not clean.  Aborting.")
    return -1

  if force is None and shout('git status --porcelain --ignored'):
    print_red("Working directory contains ignored files and/or directories.  Aborting.")
    return -1

  ecr = ECR()
  if not ecr.project_repository_exists():
    print_red("ECR repository %s does not exist... did you run `hokusai setup` for this project?" % config.project_name)
    return -1

  shout(ecr.get_login())
  if tag is None:
    tag = shout('git rev-parse HEAD').strip()

  if overwrite is None and ecr.tag_exists(tag):
    print_red("Tag %s already exists in remote repository.  Aborting." % tag)
    return -1

  docker_compose_yml = os.path.join(os.getcwd(), 'hokusai/common.yml')
  shout("docker-compose -f %s -p build build" % docker_compose_yml, print_output=True)
  build = "build_%s:latest" % config.project_name

  shout("docker tag %s %s:%s" % (build, config.aws_ecr_registry, tag))
  shout("docker push %s:%s" % (config.aws_ecr_registry, tag), print_output=True)
  print_green("Pushed %s to %s:%s" % (build, config.aws_ecr_registry, tag))

  shout("docker tag %s %s:%s" % (build, config.aws_ecr_registry, 'latest'))
  shout("docker push %s:%s" % (config.aws_ecr_registry, 'latest'), print_output=True)
  print_green("Pushed %s to %s:%s" % (build, config.aws_ecr_registry, 'latest'))



import os
import urllib

from distutils.dir_util import mkpath
from collections import OrderedDict

import yaml

from jinja2 import Environment, PackageLoader
env = Environment(loader=PackageLoader('hokusai', 'templates'))

from hokusai.lib.command import command
from hokusai.lib.config import config
from hokusai.services.ecr import ECR
from hokusai.lib.common import print_green, print_red, build_service, build_deployment, YAML_HEADER

@command
def setup(aws_account_id, project_type, project_name, aws_ecr_region, port):

  mkpath(os.path.join(os.getcwd(), 'hokusai'))

  config.create(project_name.lower().replace('_', '-'), aws_account_id, aws_ecr_region)

  if project_type == 'ruby-rack':
    dockerfile = env.get_template("Dockerfile-ruby.j2")
    base_image = 'ruby:latest'
    run_command = 'bundle exec rackup'
    development_command = 'bundle exec rackup'
    test_command = 'bundle exec rake'
    runtime_environment = {
      'development': ["RACK_ENV=development"],
      'test': ["RACK_ENV=test"],
      'production': [{'name': 'RACK_ENV', 'value': 'production'}]
    }

  elif project_type == 'ruby-rails':
    dockerfile = env.get_template("Dockerfile-rails.j2")
    base_image = 'ruby:latest'
    run_command = 'bundle exec rails server'
    development_command = 'bundle exec rails server'
    test_command = 'bundle exec rake'
    runtime_environment = {
      'development': ["RAILS_ENV=development"],
      'test': ["RAILS_ENV=test"],
      'production': [{'name': 'RAILS_ENV', 'value': 'production'},
                      {'name': 'RAILS_SERVE_STATIC_FILES', 'value': 'true'},
                      {'name': 'RAILS_LOG_TO_STDOUT', 'value': 'true'}]
    }

  elif project_type == 'nodejs':
    dockerfile = env.get_template("Dockerfile-node.j2")
    base_image = 'node:latest'
    run_command = 'node index.js'
    development_command = 'node index.js'
    test_command = 'npm test'
    runtime_environment = {
      'development': ["NODE_ENV=development"],
      'test': ["NODE_ENV=test"],
      'production': [{'name': 'NODE_ENV', 'value': 'production'}]
    }

  elif project_type == 'elixir':
    dockerfile = env.get_template("Dockerfile-elixir.j2")
    base_image = 'elixir:latest'
    run_command = 'mix run --no-halt'
    development_command = 'mix run'
    test_command = 'mix test'
    runtime_environment = {
      'development': ["MIX_ENV=dev"],
      'test': ["MIX_ENV=test"],
      'production': [{'name': 'MIX_ENV', 'value': 'prod'}]
    }

  elif project_type == 'python-wsgi':
    dockerfile = env.get_template("Dockerfile-python.j2")
    base_image = 'python:latest'
    run_command = "gunicorn -b 0.0.0.0:%s app" % port
    development_command = "python -m werkzeug.serving -b 0.0.0.0:%s %s" % (port, project_name)
    test_command = 'python -m unittest discover .'
    runtime_environment = {
      'development': ["CONFIG_FILE=config/development.py"],
      'test': ["CONFIG_FILE=config/test.py"],
      'production': [{'name': 'CONFIG_FILE', 'value': 'config/production.py'}]
    }

  with open(os.path.join(os.getcwd(), 'Dockerfile'), 'w') as f:
    f.write(dockerfile.render(base_image=base_image, command=run_command, target_port=port))

  with open(os.path.join(os.getcwd(), 'hokusai', "common.yml"), 'w') as f:
    services = {
      config.project_name: {
        'build': {
          'context': '../'
        }
      }
    }
    data = OrderedDict([
        ('version', '2'),
        ('services', services)
      ])
    payload = YAML_HEADER + yaml.safe_dump(data, default_flow_style=False)
    f.write(payload)

  for compose_environment in ['development', 'test']:
    with open(os.path.join(os.getcwd(), 'hokusai', "%s.yml" % compose_environment), 'w') as f:
      services = {
        config.project_name: {
          'extends': {
            'file': 'common.yml',
            'service': config.project_name
          }
        }
      }

      if compose_environment == 'development':
        services[config.project_name]['command'] = development_command
        services[config.project_name]['ports'] = ["%s:%s" % (port, port)]
        services[config.project_name]['volumes'] = ['../:/app']

      if compose_environment == 'test':
        services[config.project_name]['command'] = test_command

      services[config.project_name]['environment'] = runtime_environment[compose_environment]

      data = OrderedDict([
        ('version', '2'),
        ('services', services)
      ])
      payload = YAML_HEADER + yaml.safe_dump(data, default_flow_style=False)
      f.write(payload)

  for stack in ['staging', 'production']:
    with open(os.path.join(os.getcwd(), 'hokusai', "%s.yml" % stack), 'w') as f:
      if stack == 'production':
        replicas = 2
      else:
        replicas = 1

      deployment_data = build_deployment(config.project_name,
                                          "%s:%s" % (config.aws_ecr_registry, stack),
                                          port, environment=runtime_environment['production'], always_pull=True, replicas=replicas)

      service_data = build_service(config.project_name, port, target_port=port, internal=False)

      stack_yaml = deployment_data + service_data

      f.write(stack_yaml)

  print_green("Config created in ./hokusai")

  ecr = ECR()
  if ecr.project_repository_exists():
    print_green("ECR repository %s found. Skipping creation." % config.project_name)
    return 0

  if ecr.create_project_repository():
    print_green("Created ECR repository %s" % config.project_name)
    return 0
  else:
    print_red("Could not create ECR repository %s - check your credentials." % config.project_name)
    return 1



from hokusai.lib.command import command
from hokusai.lib.common import print_green
from hokusai.services.deployment import Deployment

@command
def refresh(context):
  deployment = Deployment(context)
  deployment.refresh()



import os
from setuptools import setup

setup(name='hokusai',
      version=open(os.path.join(os.path.dirname(os.path.abspath(__file__)), 'hokusai', 'VERSION.txt'), 'r').read(),
      description='Artsy Docker development toolkit',
      url='http://github.com/artsy/hokusai',
      author='Isac Petruzzi',
      author_email='isac@artsymail.com',
      license='MIT',
      packages=['hokusai'],
      package_data={'hokusai': ['VERSION.txt', 'templates/*', 'lib/*', 'services/*', 'commands/*']},
      install_requires=[
          'click==6.6',
          'MarkupSafe==0.23',
          'Jinja2==2.8',
          'PyYAML==3.12',
          'termcolor==1.1.0',
          'boto3==1.4.4',
          'botocore==1.5.41'
      ],
      zip_safe=False,
      include_package_data = True,
      scripts = ['bin/hokusai'])



from subprocess import Popen

from hokusai.lib.command import command
from hokusai.lib.config import config
from hokusai.lib.common import shout
from hokusai.services.kubectl import Kubectl

@command
def logs(context, timestamps, nlines, follow):
  kctl = Kubectl(context)

  opts = ''
  if timestamps:
    opts += ' --timestamps'
  if nlines:
    opts += " --tail=%s" % nlines
  if follow:
    opts += ' --follow'

  pods = kctl.get_object('pod', selector="app=%s,layer=application" % config.project_name)
  pods = filter(lambda pod: pod['status']['phase'] == 'Running', pods)
  containers = []
  for pod in pods:
    for container in pod['spec']['containers']:
      containers.append({'pod': pod['metadata']['name'], 'name': container['name']})

  if follow:
    processes = [Popen(kctl.command("logs %s %s%s" % (container['pod'], container['name'], opts)), shell=True) for container in containers]
    try:
      for p in processes:
        p.wait()
    except KeyboardInterrupt:
      for p in processes:
        p.terminate()
  else:
    for container in containers:
      shout(kctl.command("logs %s %s%s" % (container['pod'], container['name'], opts)), print_output=True)



import os

from collections import OrderedDict

import yaml

from hokusai.lib.command import command
from hokusai.lib.config import config
from hokusai.lib.common import print_red, print_green, shout
from hokusai.services.ecr import ECR
from hokusai.services.kubectl import Kubectl

@command
def stack_create(context):
  kubernetes_yml = os.path.join(os.getcwd(), "hokusai/%s.yml" % context)
  if not os.path.isfile(kubernetes_yml):
    print_red("Yaml file %s does not exist for given context." % kubernetes_yml)
    return -1

  ecr = ECR()
  if not ecr.project_repository_exists():
    print_red("ECR repository %s does not exist... did you run `hokusai setup` for this project?" % config.project_name)
    return -1

  if not ecr.tag_exists('latest'):
    print_red("Image tag 'latest' does not exist... did you run `hokusai push`?")
    return -1

  if not ecr.tag_exists(context):
    ecr.retag('latest', context)
    print_green("Updated tag 'latest' -> %s" % context)

  kctl = Kubectl(context)
  shout(kctl.command("create --save-config -f %s" % kubernetes_yml), print_output=True)
  print_green("Created stack %s" % context)

@command
def stack_update(context):
  kubernetes_yml = os.path.join(os.getcwd(), "hokusai/%s.yml" % context)
  if not os.path.isfile(kubernetes_yml):
    print_red("Yaml file %s does not exist for given context." % kubernetes_yml)
    return -1

  kctl = Kubectl(context)
  shout(kctl.command("apply -f %s" % kubernetes_yml), print_output=True)
  print_green("Updated stack %s" % context)

@command
def stack_delete(context):
  kubernetes_yml = os.path.join(os.getcwd(), "hokusai/%s.yml" % context)
  if not os.path.isfile(kubernetes_yml):
    print_red("Yaml file %s does not exist for given context." % kubernetes_yml)
    return -1

  kctl = Kubectl(context)
  shout(kctl.command("delete -f %s" % kubernetes_yml), print_output=True)
  print_green("Deleted stack %s" % context)

@command
def stack_status(context):
  kctl = Kubectl(context)
  print('')
  print_green("Deployments")
  print_green('-----------------------------------------------------------')
  shout(kctl.command("get deployments --selector app=%s -o wide" % config.project_name), print_output=True)
  print('')
  print_green("Services")
  print_green('-----------------------------------------------------------')
  shout(kctl.command("get services --selector app=%s -o wide" % config.project_name), print_output=True)
  print('')
  print_green("Pods")
  print_green('-----------------------------------------------------------')
  shout(kctl.command("get pods --selector app=%s -o wide" % config.project_name), print_output=True)
